#ifndef __DEFAULT_H__
#define __DEFAULT_H__

#include "Cvar_Hook.h"

void FN_StartFrame(void)
{
	if( !Cvars.size() )
	{
		RETURN_META(MRES_IGNORED);
	}

	int iReturn;
	for( size_t i = 0; i < Cvars.size(); i++ )
	{
		if( Cvars[i]->func == -1 ) continue;

		if( Cvars[i]->Value.compare(Cvars[i]->ptr->string) )
		{
			iReturn = MF_ExecuteForward(Cvars[i]->func, reinterpret_cast<cell>(Cvars[i]->ptr), Cvars[i]->Value.c_str(), (const char *)Cvars[i]->ptr->string );

			if( iReturn )
			{
				(*g_engfuncs.pfnCvar_DirectSet)(Cvars[i]->ptr, (char *)Cvars[i]->Value.c_str());
			}
			else
			{
				Cvars[i]->Value.assign(Cvars[i]->ptr->string);
			}
		}
	}

	RETURN_META(MRES_IGNORED);
}

static cell AMX_NATIVE_CALL pcvar_hook_change(AMX *amx, cell *params)
{
	cvar_t *ptr = reinterpret_cast<cvar_t *>(params[1]);
	if (!ptr)
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "Invalid CVAR pointer");
		return -1;
	}

	char* handler = MF_GetAmxString(amx, params[2], 0, NULL);
	int func = MF_RegisterSPForwardByName(amx, handler, FP_CELL, FP_STRING, FP_STRING, FP_DONE);
	
	if (func == -1)
	{
		MF_LogError(amx, AMX_ERR_NOTFOUND, "Invalid function \"%s\"", handler);
		return -1;
	}

	// Is there mem of cvars null?
	for( size_t i = 0; i < Cvars.size(); i++ )
	{
		if( Cvars[i]->func == -1 )
		{
			Cvars[i]->ptr = ptr;
			Cvars[i]->Value.assign( ptr->string );
			Cvars[i]->func = func; // Last to Prevent bug

			return i + 1;
		}
	}		

	CvarInfo *cvar = new CvarInfo;

	cvar->func = func;
	cvar->ptr = ptr;
	cvar->Value.assign( ptr->string );

	Cvars.push_back(cvar);

	return Cvars.size();
}

static cell AMX_NATIVE_CALL cvar_hook_change(AMX *amx, cell *params)
{
	char* cvtemp = MF_GetAmxString(amx, params[1], 0, NULL);

	cvar_t *ptr = CVAR_GET_POINTER(cvtemp);
	if (!ptr)
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "Invalid CVAR pointer");
		return 0;
	}

	char* handler = MF_GetAmxString(amx, params[2], 1, NULL);
	int func = MF_RegisterSPForwardByName(amx, handler, FP_CELL, FP_STRING, FP_STRING, FP_DONE);
	
	if (func == -1)
	{
		MF_LogError(amx, AMX_ERR_NOTFOUND, "Invalid function \"%s\"", handler);
		return 0;
	}

	// Is there mem of cvars null?
	for( size_t i = 0; i < Cvars.size(); i++ )
	{
		if( Cvars[i]->func == -1 )
		{
			Cvars[i]->ptr = ptr;
			Cvars[i]->Value.assign( ptr->string );
			Cvars[i]->func = func; // Last to Prevent bug

			return i + 1;
		}
	}		

	CvarInfo *cvar = new CvarInfo;

	cvar->func = func;
	cvar->ptr = ptr;
	cvar->Value.assign( ptr->string );

	Cvars.push_back(cvar);

	return Cvars.size();
}

static cell AMX_NATIVE_CALL disa_hook_change(AMX *amx, cell *params)
{
	if( params[1] <= 0 || (int)Cvars.size() < params[1] )
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "Invalid array handle provided (%d)", params[1]);
		return 0;
	}

	CvarInfo *cvar = Cvars[params[1]-1];

	if( cvar->func == -1 )
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "Invalid array handle provided (%d)", params[1]);
		return 0;
	}

	MF_UnregisterSPForward(cvar->func);
	cvar->func = -1;
	cvar->Value.empty();

	return 1;
}

AMX_NATIVE_INFO CvarHookNatives[] =
{
	{ "pcvar_hook_change",			pcvar_hook_change },
	{ "cvar_hook_change",			cvar_hook_change },
	{ "disable_hook_change",		disa_hook_change },

	{ NULL,							NULL }
};

void FN_AMXX_ATTACH(void)
{
	MF_AddNatives(CvarHookNatives);
}


#endif // __DEFAULT_H__